/*
 * main.cpp - main-file for iTALC-Application
 *
 * iTALC
 * Copyright (c) 2004-2005 Tobias Doerffel <tobydox@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program (see COPYING); if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 */


#include <qapplication.h>
#include <qtranslator.h>
#include <qtextcodec.h>
#include <qbitmap.h>
#if QT_VERSION >= 0x030200
#include <qsplashscreen.h>
#endif

#include "italc.h"
#include "embed.h"
#include "rfb_connection.h"



#if QT_VERSION >= 0x030200
QSplashScreen * splashScreen = NULL;
#endif

QString MASTER_HOST;



// good old main-function... initializes qt-app and starts iTALC
int main( int argc, char * * argv )
{
	QApplication app( argc, argv );
#if QT_VERSION >= 0x030200
	QPixmap splash = embed::getIconPixmap( "splash" );
	QPixmap actual_splash = QPixmap::grabWindow(
					QApplication::desktop()->winId(),
		( QApplication::desktop()->availableGeometry().width() -
							splash.width() ) / 2,
		( QApplication::desktop()->availableGeometry().height() -
							splash.height() ) / 2,
					splash.width(), splash.height() );
	splashScreen = new QSplashScreen( actual_splash );
	splashScreen->setMask( splash.createHeuristicMask() );
	splashScreen->show();

#ifdef ENABLE_FADEING_SPLASH
	// ok, let's have a nice fade-in-effect ;-)
	QImage splash_img = splash.convertToImage();
	QImage actual_splash_img = actual_splash.convertToImage();
	QImage dest_img = actual_splash.convertToImage();
	QTime t;
	t.start();
	while( t.elapsed() < 2000 && splashScreen->isShown() )
	{
		short int i = 256*t.elapsed()/1500;
		if( i > 256 )
		{
			i = 256;
		}

		memcpy( dest_img.bits(), actual_splash_img.bits(),
			actual_splash_img.width() * actual_splash_img.height() *
									4 );
		register uchar * rs = splash_img.bits();
		register uchar * gs = rs + 1;
		register uchar * bs = gs + 1;
		register uchar * as = bs + 1;
		register uchar * rd = dest_img.bits();
		register uchar * gd = rd + 1;
		register uchar * bd = gd + 1;
		for( short int y = 0; y < dest_img.height(); ++y )
		{
			for( short int x = 0; x < dest_img.width(); ++x )
			{
				short int opac =
	static_cast<short int>( i * (short int) *as / 256 );
				*rd += ( *rs - *rd ) * opac / 256;
				rs += 4; rd += 4;
				*gd += ( *gs - *gd ) * opac / 256;
				gs += 4; gd += 4;
				*bd += ( *bs - *bd ) * opac / 256;
				bs += 4; bd += 4;
				as += 4;
			}
		}
		// conversion is slow as hell if desktop-depth != 24bpp...
		actual_splash.convertFromImage( dest_img );
		splashScreen->setPixmap( actual_splash );
	}
#else
	bitBlt( &actual_splash, 0, 0, &splash, 0, 0, splash.width(),
						splash.height(), Qt::XorROP );
	splashScreen->setPixmap( actual_splash );
#endif

#endif

	// load translations
	QString loc = QString( QTextCodec::locale() ).section( '_', 0, 0 );
	// load translation for Qt-widgets/-dialogs
	embed::loadTranslation( QString( "qt_" ) + loc );
	// load actual translation for iTALC
	embed::loadTranslation( loc );

	// read private key, create it if it doesn't exist yet
	rfbConnection::initializeAuthentication();

	// now create the main-window
	italc * italc_main_window = italc::inst();
#if QT_VERSION >= 0x030200
	// hide splash-screen as soon as main-window is shown
	splashScreen->finish( italc_main_window );
#endif
	app.setMainWidget( italc_main_window );
	italc_main_window->show();

	// let's rock!!
	return( app.exec() );
}

